var jmaURI = "http://www.jma.go.jp/jp/typh/";
var widgetURI = "http://www2.atwiki.jp/macstar/pages/17.html";
var httpReq;
var typhNum;					// 地図を表示中の台風
var typhs = new Array();		// 台風の名前
var typhInfos = new Array();	// 台風の詳細情報（未実装）
var lastGetTime;				// 表示に使っているページの Last-Modified
var lastCheckTime;				// 最後にリロードを試みた時刻

// リサイズ関連
var lastX;
var lastY;
var width;
var height;
var minWidth = 230;
var minHeight = 210;
var maxWidth = 510;
var maxHeight = 420;

// 定期リロード関連（30分毎にリロード）
var lastCheckTime;
var timerID;
var interval = 30*60*1000;

// widget初期化
function init() {
	if (window.widget) {
		widget.onshow = onShow;
	}

	// XMLHttpRequest
	httpReq = new XMLHttpRequest();
	httpReq.onreadystatechange = processReqChange;

	typhNum = 0;
	typhs.length = 0;
	lastCheckTime = 0;
	lastGetTime = "Thu, 01 Jan 1970 00:00:00 GMT";
	mapDisp(typhNum);

	// リサイズ関連初期設定
	lastX = 0;
	lastY = 0;
	width = 270;
	height = 240;

	// 裏面の Done ボタン
	createGenericButton (document.getElementById("done"), "完了", hidePrefs);

	// 初回の読み込み
	reload();

	// 定期リロード開始
	timerID = setInterval("reload()", interval);
}


// Dashboardレイヤー表示時
function onShow() {
	var now = (new Date).getTime();

	// スリープ対策
	// 前回リロード時からinterval-1秒以上経過していたらリロード
	if ((lastCheckTime != 0) &&
			(-1*1000 < (now - lastCheckTime - interval))) {
		reload();
	}
}


// 指定された番号の地図を表示
function mapDisp(num) {
	var i;
	for (i=0; i < 8; i++) {
		if (i == num) {
			document.getElementById("map"+i).style.display = "block";
			if (document.getElementById("typh"+i))
				document.getElementById("typh"+i).style.opacity = 1.0;
		} else {
			document.getElementById("map"+i).style.display = "none";
			if (document.getElementById("typh"+i))
				document.getElementById("typh"+i).style.opacity = 0.3;
		}
	}
	typhNum = num;
}


// 最新の情報で再描画
function reload() {
	var uri = jmaURI + 'index.html';	// 台風情報トップページ
	
	// デバッグ用
	document.getElementById("status").innerText = "";
	document.getElementById("header").innerText = "";
	document.getElementById("source").innerText = "";

	// Dashboardが用意するキャッシュを使いたくないので（？）
	// 強制的にリロードさせるための小細工（URIに「?ユニークな文字列」を付加）
	uri += "?" + new Date().getTime();

//	alert ("Reload URI: " + uri );

	// HTTPリクエスト（非同期読み込み）
	// lastGetTimeよりも新しいコンテンツを要求する
	httpReq.open("GET", uri, true);
	if (0 < lastGetTime.length)
		httpReq.setRequestHeader("If-Modified-Since", lastGetTime);
	httpReq.send(null);
	
	lastCheckTime = (new Date).getTime();
}


// リクエスト状態（readyState）が変わったときに呼び出される
function processReqChange() {
	var buffer = new Array();
	var httpHeader = "";
	var htmlDoc = "";
	var i;
	var sfx;

//	alert("httpReq.readyState " + httpReq.readyState);

	// デバッグ用
	if (httpReq.readyState != 0) { // readyState 0 = uninitialized
		document.getElementById("status").innerText = httpReq.status;
		document.getElementById("header").innerText = httpReq.getAllResponseHeaders();
		document.getElementById("source").innerText = httpReq.responseText;
	}

	if (httpReq.readyState == 4) { // readyState 4 = complete
		if (httpReq.status == 200) {						// 200:ダウンロード成功
			httpHeader = httpReq.getAllResponseHeaders();	// レスポンスヘッダ
			htmlDoc = httpReq.responseText;					// HTMLソース
			document.getElementById("source").innerText = htmlDoc;

			// 最終更新時刻を記録
			if (httpHeader.match(/Last-Modified: (.+)/i))
				lastGetTime = RegExp.$1;
			else
				lastGetTime = "";

			// 台風の名前を取得 (例) all, 0504, 0505, ...
			typhs.length = 0;
			buffer.length = 0;
			buf = htmlDoc.substring(htmlDoc.indexOf("SUB_OPTION_LIST"));
			buffer = buf.match(/<option[^>]+>/g);
			if (0 < buffer.length) {
				typhs.push("index");
				for (i=0; i < buffer.length; i++) {
					if (buffer[i].match(/value=\"([0-9][0-9][0-9][0-9])\.html\"/))
						typhs.push(RegExp.$1);
				}
			}

			// 画像の拡張子を取得 png or jpg
			if (htmlDoc.match(/src=\"\.\/images\/wide\/all\.([a-zA-Z]+)\"/))
				sfx = RegExp.$1;
			else
				sfx = "png";
			
			reloadImg(sfx);
		}

		dispTyphs();
		mapDisp(typhNum);
	}
}


function reloadImg(sfx) {
	var imgURI;
	var i;

	for (i=0; i < typhs.length; i++) {
                if ( typhs.length == 1) {
                        imgURI = jmaURI + "polar_no_typh.png"; // 台風がない場合の画像
                } else {

			if (i==0) imgURI = jmaURI + "images/wide/all." + sfx;
			else imgURI = jmaURI + "images/zoom/" + typhs[i] + "." + sfx;
			// 強制的にリロードさせるための小細工
			imgURI += "?" + new Date().getTime();
		}
		document.getElementById("map"+i).setAttribute("src", imgURI);
	}
}


// 台風セレクタの表示
function dispTyphs() {
	var typhsField = document.getElementById("typhs");
	var child;
	var typhName = "";
	var i;

	while (typhsField.hasChildNodes())
		typhsField.removeChild(typhsField.lastChild);

	for (i=0; i < typhs.length; i++) {
		if (typhs[i] == "index") {
			typhName = "ALL";
		} else {
			if (typhs[i].match(/[0-9][0-9]([0-9][0-9])/)) {
				typhName = RegExp.$1;
				typhName -= 0;	// 文字列から数値に変換
			}
		}

		child = document.createElement("div");
		child.innerText = typhName;
		child.id = "typh" + i;
		child.setAttribute("onclick", "mapDisp(" + i + ");");
		typhsField.appendChild(child);
	}
}


function openWWW() {
	var uri = jmaURI + typhs[typhNum] + ".html";

	if (window.widget)
		widget.openURL(uri);
	else
		window.location.href = uri;
}

// リサイズ開始
function startDragging(event) {
	document.addEventListener("mousemove", mouseMove, true);
	document.addEventListener("mouseup", mouseUp, true);

	lastX = event.screenX;
	lastY = event.screenY;
}

// リサイズ中
function mouseMove(event) {
	var deltaX = 0;
	var deltaY = 0;

	// 最小サイズ制限
	if ((window.outerWidth + event.screenX - lastX < minWidth)
			|| (window.outerHeight + event.screenY - lastY < minHeight)) {
		window.resizeTo(minWidth, minHeight);
		resizeElements();
		return;
	}

	// 最大サイズ制限
	if ((maxWidth < window.outerWidth + event.screenX - lastX)
			|| (maxHeight < window.outerHeight + event.screenY - lastY)) {
		window.resizeTo(maxWidth, maxHeight);
		resizeElements();
		return;
	}

	deltaX = event.screenX - lastX;
	deltaY = event.screenY - lastY;

	// アスペクト比（4:3）固定
	if (window.outerHeight + deltaY - 60 < (window.outerWidth + deltaX - 30) * 0.75)
		deltaY = (window.outerWidth + deltaX - 30) * 0.75 - window.outerHeight + 60;
	else
		deltaX = (window.outerHeight + deltaY - 60) / 0.75 - window.outerWidth + 30;

	lastX = event.screenX;
	lastY = event.screenY;

	// リサイズ
	if (window.widget) {
		window.resizeBy(deltaX, deltaY);
		resizeElements();
	}
}

function resizeElements() {
	var width = window.outerWidth;
	var height = window.outerHeight;

	var maps = document.getElementById("map").childNodes;
	var front = document.getElementById("front");
	var legend = document.getElementById("legend");
	var i;

	for (i=0; i < maps.length; i++) {
		if (maps[i].nodeName == "IMG") {
			maps[i].style.width = width - 30 + "px";
			maps[i].style.height = height - 60 + "px";
		}
	}

	front.style.width = width + "px";
	front.style.height = height + "px";
	legend.style.width = width - 30 - 20 + "px";
	legend.style.height = (width -  30 - 20) / 460 * 64 + "px";
	legend.style.top = 35 + (height - 60) *0.8 + "px";
}

// リサイズ終了
function mouseUp(event) {
	document.removeEventListener("mousemove", mouseMove, true);
	document.removeEventListener("mouseup", mouseUp, true);
}

// WWW ブラウザで widget 配布ページを表示
function goWidgetPage() {
	widget.openURL(widgetURI);
}

// ひっくりかえす（Dashboard Programming Guide より）
function showPrefs() {
	if (window.widget)
		widget.prepareForTransition("ToBack");

	document.getElementById("front").style.display = "none";
	document.getElementById("back").style.display = "block";

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);  
}

function hidePrefs() {
	if (window.widget)
		widget.prepareForTransition("ToFront");

	document.getElementById("front").style.display = "block";
	document.getElementById("back").style.display = "none";

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);
}

// アニメーション関連（Dashboard Programming Guide より）
var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};

function mousemove(event) {
	document.getElementById("reload").style.display = "block";
	document.getElementById("jma").style.display = "block";
	document.getElementById("legend").style.display = "block";
	document.getElementById("growBox").style.display = "block";

	if (!flipShown) {
		if (animation.timer != null) {
			clearInterval(animation.timer);
			animation.timer = null;
		}
		var starttime = (new Date).getTime() - 13;
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById("flip");
		animation.timer = setInterval("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}

function mouseexit(event) {
	document.getElementById("reload").style.display = "none";
	document.getElementById("jma").style.display = "none";
	document.getElementById("legend").style.display = "none";
	document.getElementById("growBox").style.display = "none";

	if (flipShown) {
		// fade in the info button
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		var starttime = (new Date).getTime() - 13;
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById("flip");
		animation.timer = setInterval("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
	exitflip();
}

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();
	T = limit_3(time-animation.starttime, 0, animation.duration);
	if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	} else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	animation.firstElement.style.opacity = animation.now;
}

function limit_3 (a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
	return from + (to - from) * ease;
}

function enterflip(event) {
	document.getElementById("fliprollie").style.display = "block";
}

function exitflip(event) {
	document.getElementById("fliprollie").style.display = "none";
}
